# $Id: scriptAPIsetup.tcl,v 1.7.6.2 2007-07-20 11:57:53 pnunes Exp $
# File   : scriptAPIsetup.tcl
# Purpose: Initialize the EVM scripting environment for TCL
# Author : Nick Kornweibel
# Date   : Tue May 29 14:15:39 MET DST 2001
#
# $Id: scriptAPIsetup.tcl,v 1.7.6.2 2007-07-20 11:57:53 pnunes Exp $

# include java
package require java

#
# define the procedures that are available to the TCL script developers
#

# evmInit:
# Initialization procedure. Must be called at the beginning to set up the 
# environment: build the scripting box, initialize the data structures, etc. 
# Returns the EVM version string, for instance 1.0, that defines the version 
# of the scripting interface. The version string allows for the run-time 
# checking of the scripting interface by the plug-in script (interface 
# incompatibility is not detected until run-time). 

proc evmInit {} {
    global scriptingBox
    set scriptingBox [java::call org.eso.ohs.scripting.ScriptingBox getInstance]
    return [$scriptingBox getVersion]
}

# evmInfo:
# Add string text to the verification report. The EVM does not process text 
# in any way.

proc evmInfo description {
    global scriptingBox
    $scriptingBox infoMessage "$description"
}

#
# evmHeader:
# Override the default report header with these strings. First
# call to evmHeader replaces the default header, subsequent calls
# append to the report header.
# Report headers are printed at the start of the OB Validation report.

proc evmHeader description {
    global scriptingBox
    $scriptingBox headerMessage "$description"
}
    
#
# evmFooter:
# Override the default report footer with these strings. First
# call to evmFooter replaces the default footer, subsequent calls
# append to the report footer.
# Report footers are printed at the end of the OB Validation report.

proc evmFooter description {
    global scriptingBox
    $scriptingBox footerMessage "$description"
}

# 
# evmValid:
# Used to determine if an object returned by evm procedures is valid or not
# This should be used to check all object returned by evm procedures.

proc evmValid in {
    if { [java::isnull $in] } {
        return 0
    } else {
        return 1
    }
}

# evmWarning:
# Report a warning. String text is added to the verification report prefixed 
# by the string "warning"; the warning counter is incremented by one.

proc evmWarning description {
    global scriptingBox
    $scriptingBox warningMessage "$description"
}

# evmError:
# Report a verification error. String text is added to the verification 
# report prefixed by "error"; the error counter is incremented by one. 

proc evmError description {
    global scriptingBox
    $scriptingBox errorMessage "$description"
}

# evmResetWarningCounter:
# Tell the application that all warnings so far reported are to be cleared.

proc evmResetWarningCounter {} { 
    global scriptingBox
    $scriptingBox clearWarnings
}

# evmResetErrorCounter:
# Tell the application that all errors so far reported are to be cleared.

proc evmResetErrorCounter {} { 
    global scriptingBox
    $scriptingBox clearErrors
}

# evmGetOBCount:
# return the number of OBs loaded into the scripting environment

proc evmGetOBCount {} {
    global scriptingBox
    return [$scriptingBox getNumberOfObservationBlocks]
}

# evmGetOB:
# return the OB at the specified index (or null if the index is out of
# range). First index is 0.
# returned objects should be checked using evmValid

proc evmGetOB index {
    global scriptingBox
    return [$scriptingBox getObservationBlockAt $index]
}

# evmName:
# return the name of an object

proc evmName obj {
    return [$obj getName]
}

# evmInternalError:
# used to exit processing and return an error message
# to the top level application.

proc evmInternalError msg {
    error $msg
}

# evmID:
# return an id for the object

proc evmID obj {
    return [$obj getId]
}

# evmType:
# return the type of an OB "C" for Calibration, "O" for Observation

proc evmType ob {
    return [$ob getType]
}

# evmObservationBlocks:
# return a list of observationBlocks of the specified type

proc evmObservationBlocks { { type "All" } } {
    set rc {}
    set obcount [evmGetOBCount]
    set oid 0
    while { $oid < $obcount } {
        set ob [evmGetOB $oid]
        if { $type == "All" || $type == [evmType $ob] } {
            lappend rc $ob
        }
        incr oid 1
    }
    return $rc
}

# evmRunID:
# return the Run Id of an Observation Block

proc evmRunID ob {
    return [$ob getRunID]
}

# evmRunMode:
# return the Run Id of an Observation Block

proc evmRunMode ob {
    return [$ob getRunMode]
}


# evmInstrumentName:
# return the name of the instrument used in the OB

proc evmInstrumentName ob {
    return [$ob getInstrumentName]
}

# evmInstrumentSpecificComments:
# return the name of the instrument used in the OB

proc evmInstrumentSpecificComments ob {
    return [$ob getInstrumentComments]
}

# evmUserComments:
# return the user comments

proc evmUserComments ob {
    return [$ob getUserComments]
}

# evmTarget:
# return the target in used in the OB, or null if no target.
# returned objects should be checked using evmValid

proc evmTarget ob {
    return [$ob getTarget]
}

# evmConstraintSet:
# return the constraint set used in the ob, or null if no target.
# returned objects should be checked using evmValid

proc evmConstraintSet ob {
    return [$ob getConstraintSet]
}

# evmTimeIntervals:
# return a list of Time Intervals used in the OB.

proc evmTimeIntervals ob {
    set rc {}
    set timeIntSet [$ob getTimeIntervals]
    if { [evmValid $timeIntSet] } {
        set count [$timeIntSet getNumberOfTimeIntervals]
        set tid 0
        while { $tid < $count } {
            set t [$timeIntSet getTimeInterval $tid]
            lappend rc $t
            incr tid 1
        }
    }
    return $rc
}

# evmTimeIntervals:
# return a list of Time Intervals used in the OB.

proc evmSTTimeIntervals ob {
    set rc {}
    set timeIntSet [$ob getSTTimeIntervals]
    if { [evmValid $timeIntSet] } {
        set count [$ob getNumberOfSiderealTimeIntervals]
        set tid 0
        while { $tid < $count } {
            set t [$timeIntSet getTimeInterval $tid]
            lappend rc $t
            incr tid 1
        }
    }
    return $rc
}



# evmExposureTime:
# return the Exposure Time of an OB.

proc evmExposureTime ob {
    return [$ob getExpTime]
}

# evmExecutionTime:
# return the Execution Time of an OB.

proc evmExecutionTime ob { 
    return [$ob getExecutionTime]
}


# evmTemplates:
# return a list of OD templates used in the OB, the first template may be null
# if there is no ACQ template in the OB.
# returned objects should be checked using evmValid

proc evmTemplates ob {
    set rc {}
    set count [$ob getNumberOfODTemplates]
    set tid 0
    while { $tid <= $count } {
        set t [$ob getTemplateAt $tid]
        lappend rc $t
        incr tid 1
    }
    return $rc
}

# evmTemplateAt
# return the template at the specified index
# if there is an ACQ template, it is returned in index 0, else null
# is returned for index 0
# returned objects should be checked using evmValid

proc evmTemplateAt { ob idx } {
    return [$ob getTemplateAt $idx]
}

# evmAcquisitionTemplate:
# return the acquisition template of the OB, if there is one, else empty list
# returned objects should be checked using evmValid

proc evmAcquisitionTemplate ob {
    return [evmTemplateAt $ob 0]
}

# evmTemplateType
# Return the template type - this is taken from the parameter value TPL.TYPE.
# If the template does not contain this parameter, an empty string is returned.

proc evmTemplateType tpl {
    return [$tpl getTemplateType]
}

# evmTemplateAcquisitionTime:
# return the acquisition time for a template, or 0 if not known or
# template is not an acquisition template.

proc evmTemplateAcquisitionTime tpl {
    return [$tpl getAcquisitionTime]
}


# evmParameters:
# return a list of parameter names matching the reg exp

proc evmParameters { temp { exp ".*" } } {
    set rc {}
    set pnum [$temp getNumberOfParameters]
    set pid 0
    while { $pid < $pnum } {
        set pname [$temp getParameterNameAt $pid]
        set match [regexp $exp $pname]
        if { $match == 1 } {
            set pval [$temp getParameterNameAt $pid]
            lappend rc $pval
        }
        incr pid 1
    }
    return $rc
}

# evmValue:
# return the value for the specified param from the specified template
# if paf param is specified and the value is paf format the param value of
# the paf param with-in the value is returned
#
# This procedure should NOT be used for readin PAF values when
# many parameters are to be read from the PAF value. Using
# evmParameterValue involves parsing the PAF value for every call
# and for large PAF values will be very inefficient.
# Use evmParsePAF and evmGetPAFParameter instead.

proc evmParameterValue { tpl param { pafParam "" } } {
    global scriptingBox
    set rc [$tpl getParameterValue $param]
    if { $pafParam != "" && $rc != "" } {
        set pafTable [evmParsePAF $rc]
        set rc [evmGetPAFParameter $pafTable $pafParam]
    }
    return $rc
}

proc evmCheckPafChecksum { val } {
	set pafTable evmParsePAF $val
	set rc [$pafTable checkCRC]
	return $rc  
}

# evmParsePAF:
# return a PAFTable object built from the PAF format string
# passed in the call. The PAFTable will be empty if the string
# is not PAF format.
#
# This procedure in conjunction with evmGetPAFParameter
# should be used over evmParameterValue when
# many parameters are to be read from the PAF file. Using
# evmParameterValue involves parsing the PAF value for every call
# and for large PAF values will be very inefficient.

proc evmParsePAF { val } {
    global scriptingBox
    set utilsLib [$scriptingBox getUtils]
    set rc [$utilsLib parsePAF $val]
    return $rc
}

# evmGetPAFParameter:
# Search pafTable and return the first occurrence of the parameter
# paramName and return its value. "" is return if not located.
#
proc evmGetPAFParameter { pafTable paramName } {
    set rc ""
    set pvals [$pafTable getValuesForParam $paramName]
    if { [$pvals getElementCount] > 0 } {
        set rc [$pvals getElementAt 0]
    }
    return $rc
}

# evmRightAscension
# return target RA (in milliarcsec if specified)

proc evmRightAscension { targ { milliarc ""} } {
    set rc [ $targ getRightAscension ]
    if { $milliarc != "" } {
        global scriptingBox
        set rc [[$scriptingBox getUtils] getRAMilliseconds $rc]
    }
    return $rc
}

# evmDeclination
# return target Dec (in milliarcsec if specified)

proc evmDeclination { targ { milliarc ""} } {
    set rc [ $targ getDeclination ]
    if { $milliarc != "" } {
        global scriptingBox
        set rc [[$scriptingBox getUtils] getDecMilliarcseconds $rc]
    }
    return $rc
}
# evmPropDec
# return target proper Declicantion (in milliarcsec if specified)

proc evmPropDec { targ { milliarc ""} } {
    set rc [ $targ getPropDec ]
    if { $milliarc != "" } {
        global scriptingBox
        set rc [[$scriptingBox getUtils] getDecMilliarcseconds $rc]
    }
    return $rc
}

# evmPropRA
# return target proper RA (in milliarcsec if specified)

proc evmPropRA { targ { milliarc ""} } {
    set rc [ $targ getPropRA ]
    if { $milliarc != "" } {
        global scriptingBox
        set rc [[$scriptingBox getUtils] getDecMilliarcseconds $rc]
    }
    return $rc
}

# evmDiffDec
# return target Diff Declination (in milliarcsec if specified)

proc evmDiffDec { targ { milliarc ""} } {
    set rc [ $targ getDiffDec ]
    if { $milliarc != "" } {
        global scriptingBox
        set rc [[$scriptingBox getUtils] getDecMilliarcseconds $rc]
    }
    return $rc
}

# evmDiffRA
# return target Diff RA (in milliarcsec if specified)

proc evmDiffRA { targ { milliarc ""} } {
    set rc [ $targ getDiffRA ]
    if { $milliarc != "" } {
        global scriptingBox
        set rc [[$scriptingBox getUtils] getDecMilliarcseconds $rc]
    }
    return $rc
}

# evmEquinox:
# return the equinox of the target

proc evmEquinox targ {
    return [$targ getEquinox]
}

# evmEpoch:
# return the epoch of the target

proc evmEpoch targ {
    return [$targ getEpoch]
}

# Constraint set functions

# evmAirMass:
# return the airmass 

proc evmAirMass conset {
    return [$conset getAirmass]
}

# evmFractionalLunarIllumination:
# Return the fractional lunar illumination specified in the constraint set.

proc evmFractionalLunarIllumination conset {
    return [$conset getFractionalLunarIllumination]
}

# evmSkyTransparency:
# Return the sky transparency specified in the constraint set.

proc evmSkyTransparency conset {
    return [$conset getSkyTransparency]
}

# evmMoonAngularDistance:
# Return the moon angular distance specified in the constraint set.

proc evmMoonAngularDistance conset {
    return [$conset getMoonAngularDistance]
}

# evmStrehlRatio:
# Return the Strehl Ratio specified in the constraint set.

proc evmStrehlRatio conset {
    return [$conset getStrehlRatio]
}


# evmSeeing:
# Return the seeing specified in the constraint set.

proc evmSeeing conset {
    return [$conset getSeeing]
}

# evmTelescopeTemp:
# return the telscope temperature specified in the constraint set.

proc evmTelescopeTemp conset {
    return [$conset getTelTemp]
}

# evmWaterVapour:
# Return the water vapour level specified in the constraint set.

proc evmWaterVapour conset {
    return [$conset getWaterVapour]
}

# evmConstraintSetComments:
# Return the user comments attached to this constraint set.

proc evmConstraintSetComments conset {
    return [$conset getDescription]
}

# evmBaseline:
# Return the baseline specified in the constraint set
proc evmBaseline conset {
    return [$conset getBaseline]
} 

# Time Interval procedures

# evmStartTime:
# return the start time (in seconds) when this OB could be executed

proc evmStartTime timint {
    return [$timint getStartTime]
}

# evmEndTime:
# return the end time (in seconds) when this OB could be executed

proc evmEndTime timint {
    return [$timint getEndTime]
} 

# evmTimeIntervalPreference:
# Return (as an integer) the user specified preference for this time interval.

proc evmTimeIntervalPreference timint {
    return [$timint getPreference]
}

##### Finding Chart procedures

# evmFindingCharts:
# Returns an array of finding charts
proc evmFindingCharts ob {
    set rc {}
    set fChartSet [$ob getFindingCharts]
    if { [evmValid $fChartSet] } {
        set count [$fChartSet getNumberOfFindingCharts]
        set fcCounter 0
        while { $fcCounter < $count } {
            set fc [$fChartSet getFindingChart $fcCounter]
            lappend rc $fc
            incr fcCounter 1
        }
    }
    return $rc
}

# evmFCName:
# Returns the name of the finding chart
proc evmFCName fc {
    return [$fc getName]
}


# evmFCHeight:
# Returns the height of the finding chart in pixels
proc evmFCHeight fc {
    return [$fc getHeight]
}


# evmFCWidth:
# Returns the width of the finding chart in pixels
proc evmFCWidth fc {
    return [$fc getWidth]
}

# evmFCWidth:
# Returns the size of the finding chart in bytes
proc evmFCSize fc {
    return [$fc getSize]
}


# evmExposureTime:
# return the Exposure Time of an OB.

proc evmExposureTime ob {
    return [$ob getExposureTime]
}

# evmExecutionTime:
# return the Execution Time of an OB.

proc evmExecutionTime ob { 
    return [$ob getExecutionTime]
}

# Set the execution time of an ob

proc evmSetExecutionTime {ob exectime} {     
     $ob setExecutionTime [java::call Integer parseInt [java::new String $exectime]]     
}

# Method to set the exposure time of an OB
proc evmSetExposureTime {ob exptime} {
      $ob setExposureTime [java::call Integer parseInt [java::new String $exptime]]
}


# set the parameter value of a template.

proc evmSetParameterValue {tpl paramName paramValue} {     
      $tpl setParameterValue [java::new String $paramName] [java::new String $paramValue]
}


# Method to set the exposure time of an OB
proc evmSetExposureTime {ob exptime} {
      $ob setExposureTime [java::call Integer parseInt [java::new String $exptime]]
}

# Return the number of finding charts

proc evmGetNumberOfFindingCharts ob {
   return [$ob getNumberOfFindingCharts]
}


#
# evmTest
# Test procedure for the above functions:
# To run the test procedure, call evmTest from an evm script (calling it in this
# script is of limited use as no OBs are loaded when this initialization
# script is run)
#

proc evmTest {} {
    global scriptingBox
    set version [evmInit]
    evmInfo "TCL API Generated Report"
    evmInfo "API Version: $version"
    evmInfo "Test INFO message"
    evmWarning "Test WARNING message"
    evmError "Test ERROR message"
    evmResetWarningCounter
    evmResetErrorCounter
    evmWarning "Should be the first WARNING message"
    evmError "Should be the first ERROR message"
    set obcount [evmGetOBCount]
    set oid 0
    while { $oid < $obcount } {
        set ob [evmGetOB $oid]
        evmInfo "Name: [evmName $ob] Id: [evmID $ob]"
        evmInfo "Type: [evmType $ob] Run Id: [evmRunID $ob]"
        evmInfo "Inst: [evmInstrumentName $ob]"
        evmInfo "Exp Time: [evmExposureTime $ob]"
        evmInfo "Exec Time: [evmExecutionTime $ob]"
    
        # target
        set targ [evmTarget $ob]
        if { [evmValid $targ] } {
            evmInfo "Target Name: [evmName $targ] Target id: [evmID $targ]"
            evmInfo "Target RA           : [evmRightAscension $targ ]"
            evmInfo "Target RA (millarc) : [evmRightAscension $targ "milli" ]"
            set dec [evmDeclination $targ "milli" ]
            evmInfo "Target Declination  : [evmDeclination $targ ]"
            evmInfo "Target Dec (millarc): [evmDeclination $targ "milli" ]"
           
            evmInfo "Target Equinox      : [evmEquinox $targ ]"
            evmInfo "Target Epoch        : [evmEpoch $targ ]"
			evmInfo "Target PropRA       : [evmPropRA $targ ]"
            evmInfo "Target PropDec      : [evmPropDec $targ ]"
            evmInfo "Target DiffDec      : [evmDiffDec $targ ]"
            evmInfo "Target Equinox      : [evmEquinox $targ ]
        } else {
            evmInfo "No Target"
        }
        # constraint set
        set cs [evmConstraintSet $ob]
        if { [evmValid $cs] } {
            evmInfo "CS Name: [evmName $cs] CS id: [evmID $cs]"
            evmInfo "Air Mass         : [evmAirMass $cs]"
            evmInfo "Frac. Lunar Ill. : [evmFractionalLunarIllumination $cs]"
            evmInfo "Sky Transp.      : [evmSkyTransparency $cs]"
            evmInfo "Moon Angular Dis.: [evmMoonAngularDistance $cs]"
            evmInfo "Seeing           : [evmSeeing $cs]"
            evmInfo "Tele. Temp       : [evmTelescopeTemp $cs]"
            evmInfo "Water Vapour     : [evmWaterVapour $cs]"
            evmInfo "Comments         : [evmConstraintSetComments $cs]"
            evmInfo "Strehl Ratio     : [evmStrehlRatio $cs]"
        } else {
            evmInfo "No Constraint Set"
        }

        # time interval set
        set tis [evmTimeIntervals $ob]
        if { [llength $tis] } {
            foreach ti $tis {
                evmInfo "TI Name      : [evmName $ti] Ti id: [evmID $ti]"
                evmInfo "Start time   : [evmStartTime $ti]"
                evmInfo "End time     : [evmEndTime $ti]"
                evmInfo "Preference   : [evmTimeIntervalPreference $ti]"
            }
        } else {
            evmInfo "No time intervals"
        }
    
        # templates
        set templates [evmTemplates $ob]
        set tid 0
        foreach temp $templates {
            if { [evmValid $temp] } {
                evmInfo "Template $tid name: [evmName $temp]"
                evmInfo "Template $tid id: [evmID $temp]"
                evmInfo "Template $tid type: [evmTemplateType $temp]"
                evmInfo "Template $tid acq time: [evmTemplateAcquisitionTime $temp]"
                evmInfo "Template $tid exp time: [evmTemplateExposureTime $temp]"
                evmInfo "Template $tid exec time: [evmTemplateExecutionTime $temp]"

                # get all parameters starting with DET
                set exp "^DET"
                evmInfo "Template $tid DET params: [evmParameters $temp $exp ]"

                # get the INS.SLIT.NO paf-param from the INS.GBR.SETUP param
                set pnm "INS.GBR.SETUP"
                set pfnm "INS.SLIT.NO"
                evmInfo "PAF $pnm $pfnm=[evmParameterValue $temp $pnm $pfnm]"

                # loop through all params and print the names out
                set pid 0
                foreach param [evmParameters $temp] {
                    incr pid 1
                    set pval [evmParameterValue $temp $param]
                    # only want to print short value strings ...
                    if { [string length $pval] > 25 } {
                        set pval "[string range $pval 0 20]..."
                    }
                    evmInfo "Param $pid: $param $pval"
                }
            } else {
                evmInfo "Empty template at idx: $tid"
            }
            incr tid 1
        }
    
        incr oid 1
    }
}

# end of file scriptAPIsetup.tcl
